# -*- coding: utf-8 -*-
import ldtools
import rdflib
import unittest2
from rdflib import compare


class TestOriginGET(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

        uri = "http://www.w3.org/People/Berners-Lee/card"
        self.origin, created = ldtools.Origin.objects.get_or_create(uri)
        self.origin.GET()


class TestResource2(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

    def test_manager_get_origin_guessing(self):
        uri = "http://example.org/resource"
        resourceuri = uri + "#me"
        resource, created = ldtools.Resource.objects.get_or_create(resourceuri,
                                                          auto_origin=True)
        assert str(resource._uri) == resourceuri
        resource_get = ldtools.Resource.objects.get(resourceuri)

    def test_manager_get_origin_guessing_miss(self):
        uri = "http://example.org/resource"
        resourceuri = uri + "#me"
        self.assertRaises(ldtools.Resource.DoesNotExist,
            ldtools.Resource.objects.get, resourceuri)


if __name__ == '__main__':
    unittest2.main()


import rdflib


class DoesNotExist(Exception):
    "The requested object does not exist"
    silent_variable_failure = True


class MultipleObjectsReturned(Exception):
    "More than one object exists --> inconsistency"
    silent_variable_failure = True


class Field(object):
    def to_db(self, value=None):
        if value is None:
            value = ''
        return value
    def to_python(self, value=None):
        return value


class StringField(Field):
    def to_python(self, value=None):
        return unicode(value) or u""


class URIRefField(Field):
    def to_python(self, value=None):
        if value:
            assert not isinstance(value, rdflib.Literal)
            assert isinstance(value, (rdflib.URIRef,
                rdflib.BNode)), "please pass uriref!"
            #value = rdflib.URIRef(unicode(value))
        else:
            value = rdflib.URIRef(u'')
        return value
class ObjectField(Field):
    def to_python(self, value=None):
        return value


class Options(object):

    def __init__(self, meta, attrs):
        fields = []
        for obj_name, obj in attrs.iteritems():
            if isinstance(obj, Field):
                fields.append((obj_name, obj))
        self.fields = dict(fields)


class ModelMeta(type):

    def __new__(cls, name, bases, attrs):
        super_new = super(ModelMeta, cls).__new__
        parents = [b for b in bases if isinstance(b, ModelMeta)]
        if not parents:
            # If this isn't a subclass of Model, don't do anything special.
            return super_new(cls, name, bases, attrs)

        module = attrs.pop('__module__')

        new_cls = super_new(cls, name, bases, {'__module__': module})

        attr_meta = attrs.pop('Meta', None)
        if not attr_meta:
            meta = getattr(new_cls, 'Meta', None)
        else:
            meta = attr_meta
        setattr(new_cls, '_meta', Options(meta, attrs))

        # Add all attributes to the class.
        for obj_name, obj in attrs.iteritems():
            if isinstance(obj, Manager):
                obj.contribute_to_class(new_cls, obj_name)
            else:
                setattr(new_cls, obj_name, obj)

        if not hasattr(new_cls, "__unicode__"):
            new_cls.__unicode__ = lambda self: self.pk
        if not hasattr(new_cls, '__str__'):
            new_cls.__str__ = lambda self: self.__unicode__()
        new_cls.__repr__ = lambda self: u'<%s: %s>' % (
                self.__class__.__name__, self.__unicode__())
        return new_cls


class Model(object):

    __metaclass__ = ModelMeta
    MultipleObjectsReturned = MultipleObjectsReturned
    DoesNotExist = DoesNotExist

    def __init__(self, pk=None, **kwargs):
        self.pk = pk
        for attr_name, field in self._meta.fields.iteritems():
            attr = kwargs.pop(attr_name)
            value = field.to_python(attr)
            setattr(self, attr_name, value)
        if kwargs:
            # TODO ValueError is raised on object creation with kwargs.
            # inconsistent: after that, no checks performed.
            raise ValueError('%s are not part of the schema for %s'
                % (', '.join(kwargs.keys()), self.__class__.__name__))

    def __eq__(self, other):
        if not type(other) == type(self):
            return False

        if set(other.__dict__.keys()) != set(self.__dict__.keys()):
            return False

        for key in self.__dict__.keys():
            if (not hasattr(other, key) or
                    getattr(self, key) != getattr(other, key)):
                return False

        return True


    def __ne__(self, other):
        if type(other) != type(self):
            return True

        if set(other.__dict__.keys()) != set(self.__dict__.keys()):
            return True

        for key in self.__dict__.keys():
            if (not hasattr(other, key) or
                    getattr(other, key) != getattr(self, key)):
                return True

        return False

    def __hash__(self):
        # It is wrong to really compare the object here. This case is
        # important to work with references in set() instances for instance
        return hash(self.pk)


class ManagerDescriptor(object):
    # This class ensures managers aren't accessible via model instances.
    # Poll.objects works, but poll_obj.objects raises AttributeError.
    def __init__(self, manager):
        self.manager = manager

    def __get__(self, instance, type=None):
        if instance != None:
            raise AttributeError("Manager isn't accessible via %s instances"
                                 % type.__name__)
        return self.manager


class Manager(object):

    def __init__(self):
        self.model = None
        self.reset_store()

    def contribute_to_class(self, model, name):
        self.model = model
        setattr(model, name, ManagerDescriptor(self))

    def reset_store(self):
        self._storage = {}

    def all(self):
        return self._storage.values()

    def filter_has_key(self, key):
        key = unicode(key)
        func = lambda obj: key in [unicode(k) for k in \
                                        obj.__dict__.keys()]
        return filter(func, self.all())

    def filter(self, **kwargs):
        def check_if_equals_or_in_set((key, value)):
            """helper method for loop. Convenient but maybe hacky: checks
            if value is in attr or if iterable inside the iterable"""
            if hasattr(item, key):
                items_value = getattr(item, key)
                # TODO why can't we check for hasattr(items_value, "__iter__")?
                if type(items_value) in [list, set]:
                    if value in items_value:
                        return True
                else:
                    if unicode(items_value) == unicode(value):
                        return True
            return False

        for item in self.all():
            if all(map(check_if_equals_or_in_set, kwargs.items())):
                yield item

    def create(self, pk, **kwargs):
        instance = self.model(pk=pk, **kwargs)
        assert not pk in self._storage, ("%s object with pk %s already exists!"
            % (self.model, pk))
        self._storage[pk] = instance
        return instance

    def get(self, pk):
        if pk in self._storage:
            return self._storage[pk]
        else:
            raise self.model.DoesNotExist



# -*- coding: utf-8 -*-
import ldtools
import rdflib
import unittest2
import urlparse


class TestUrlparse(unittest2.TestCase):

    def test_hash1(self):
        uri = "www.cwi.nl:80/%7Eguido/Python.html"
        o = urlparse.urlparse(uri)
        self.assert_(not o.fragment)

    def test_hash2(self):
        uri = "www.cwi.nl:80/%7Eguido/Python.html#fragm"
        o = urlparse.urlparse(uri)
        self.assertEqual(o.fragment, "fragm", o.fragment)

    def test_hash3(self):
        uri = "www.cwi.nl:80/%7Eguido/Python.html#bla"
        r = urlparse.urlsplit(uri)
        self.assertEqual(r.geturl(), "www.cwi.nl:80/%7Eguido/Python.html#bla",
            r.geturl())


class TestCanonalizeUri(unittest2.TestCase):

    def test_empty1(self):
        self.assertRaises(ldtools.UriNotValid, ldtools.canonalize_uri, "")

    def test_empty2(self):
        self.assertRaises(ldtools.UriNotValid, ldtools.canonalize_uri,
                          rdflib.URIRef(""))

    def test_literal(self):
        # TODO
        self.assertRaises(AssertionError, ldtools.canonalize_uri,
                          rdflib.Literal("a"))

    def test_relative(self):
        self.assertRaises(ldtools.UriNotValid, ldtools.canonalize_uri,
                          rdflib.URIRef("#me"))

    def test_works(self):
        uri = rdflib.URIRef("http://web.de/test?query=bla")
        self.assert_(ldtools.canonalize_uri(uri), uri)


class TestRdflibLiterals(unittest2.TestCase):

    def test_1(self):
        self.assertNotEquals(rdflib.Literal('TimBL'), rdflib.Literal('timbl'))
        self.assertNotIn(rdflib.Literal('TimBL'),
                         set([rdflib.Literal('timbl')]))
        self.assertIn(rdflib.Literal('timbl'), set([rdflib.Literal('timbl')]))


if __name__ == '__main__':
    unittest2.main()


# -*- coding: utf-8 -*-
import ldtools
import rdflib
import unittest2
from rdflib import compare


class TestOrigin(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

    def test_manager_create_and_get(self):
        # Sorry @timbl that I use your bandwidth here. Ideas for better example?
        uri = "http://www.w3.org/People/Berners-Lee/card"
        ldtools.Origin.objects.create(uri)
        r = ldtools.Origin.objects.get(uri)
        self.assertIn(r, ldtools.Origin.objects.all())
        self.assertEquals(str(r.uri), uri)


class TestOriginGET(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

        # Sorry @timbl that I use your bandwidth here. Ideas for better example?
        uri = "http://dbpedia.org/resource/Karlsruhe"
        self.origin, created = ldtools.Origin.objects.get_or_create(uri)
        self.origin.GET()

    def test_graphs_equal(self):
        g1 = self.origin._graph
        g2 = self.origin.graph()

        # normal rdflib.compare does not work correctly with
        # conjunctiveGraph, unless there is only one graph within that
        self.assert_(len(list(g1.contexts())), 1)
        self.assert_(len(list(g2.contexts())), 1)

        self.assert_(len(g1), len(g2))
        self.assert_(compare.to_isomorphic(g1) == compare.to_isomorphic(g2))

    def test_graphs_namespace_bindings_equal(self):
        nsbindungs_orig = dict(self.origin._graph.namespace_manager\
            .namespaces())
        nsbindungs_new = dict(self.origin.graph().namespace_manager\
            .namespaces())
        self.assertEqual(nsbindungs_orig, nsbindungs_new)

    def test_get_all(self):
        self.origin.__class__.objects.GET_all()


class TestResource1(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()
        uri = "http://example.org/resource"
        self.origin = ldtools.Origin.objects.create(uri)

    def test_manager_create(self):
        resource = ldtools.Resource.objects.create("#me", origin=self.origin)
        assert str(resource._uri) == self.origin.uri + "#me"
        assert len(ldtools.Resource.objects.all()) == 1

    def test_manager_get(self):
        uri = "http://example.com"
        resource = ldtools.Resource.objects.create(uri, origin=self.origin)
        resource_get = ldtools.Resource.objects.get(uri, origin=self.origin)
        self.assert_(resource)
        self.assertEqual(resource, resource_get)

    def test_manager_filter(self):
        uri = "http://example.com"
        resource = ldtools.Resource.objects.create(uri, origin=self.origin)
        resource2 = ldtools.Resource.objects.create(uri+"/test",
                                                    origin=self.origin)
        resource_filter = ldtools.Resource.objects.filter(_origin=self.origin)
        self.assertIn(resource, resource_filter)

    def test_manager_create_auto_origin(self):
        uri = "http://example.org/resource#me"
        resource, created = ldtools.Resource.objects.get_or_create(uri,
                                                          auto_origin=True)
        self.assert_(str(resource._origin.uri), uri.rstrip('#me'))
        self.assert_(len(ldtools.Resource.objects.all()), 1)
        self.assert_(len(ldtools.Resource.objects.all()), 1)


class TestResource2(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

    def test_manager_get_origin_guessing(self):
        uri = "http://example.org/resource"
        resourceuri = uri + "#me"
        resource, created = ldtools.Resource.objects.get_or_create(resourceuri,
                                                          auto_origin=True)
        assert str(resource._uri) == resourceuri
        resource_get = ldtools.Resource.objects.get(resourceuri)

    def test_manager_get_origin_guessing_miss(self):
        uri = "http://example.org/resource"
        resourceuri = uri + "#me"
        self.assertRaises(ldtools.Resource.DoesNotExist,
            ldtools.Resource.objects.get, resourceuri)

if __name__ == '__main__':
    unittest2.main()


# -*- coding: utf-8 -*-
import ldtools
import os
import rdflib
import unittest2
from rdflib import compare


class TestOriginGET(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

        self.foaf_uri1 = "http://xmlns.com/foaf/0.1/"
        self.foaf_uri2 = "http://example.org/"

        self.sample_data = """<rdf:RDF
      xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
      xmlns:foaf="%s">
    <foaf:Person rdf:about="#me">
        <foaf:name>Max Mustermann</foaf:name>
    </foaf:Person></rdf:RDF>"""

    def _scenario_setup(self, foaf_uri1, foaf_uri2):
        data = self.sample_data % foaf_uri1
        backend = ldtools.MemoryBackend(data=data)
        origin1 = ldtools.Origin.objects.create("http://example.org/sample1",
                                                     BACKEND=backend)
        origin1.GET()

        data = self.sample_data % foaf_uri2
        backend = ldtools.MemoryBackend(data=data)
        origin2 = ldtools.Origin.objects.create("http://example.org/sample2",
                                                     BACKEND=backend)
        origin2.GET()

    def test_check_shortcut_consistency_does_not_complain(self):
        self._scenario_setup(self.foaf_uri1, self.foaf_uri1)
        self.assert_(not ldtools.check_shortcut_consistency())

    def test_check_shortcut_consistency_complains(self):
        self._scenario_setup(self.foaf_uri1, self.foaf_uri2)
        self.assertRaises(AssertionError, ldtools.check_shortcut_consistency)

if __name__ == '__main__':
    unittest2.main()


# -*- coding: utf-8 -*-

import copy
import datetime
import logging
import mimetypes
import rdflib
import socket;
from rdflib.namespace import split_uri
from rdflib import compare
from urlparse import urlparse
from xml.sax._exceptions import SAXParseException

import utils

# TODO: find a better way to set socket timeout
socket.setdefaulttimeout(5)

logger = logging.getLogger(__name__)


class UriNotValid(Exception):
    "Given Uri is not valid"
    silent_variable_failure = True


def safe_dict(d):
    """Recursively clone json structure with UTF-8 dictionary keys"""
    if isinstance(d, dict):
        return dict([(k.encode('utf-8'), safe_dict(v))
                     for k,v in d.iteritems()])
    elif isinstance(d, list):
        return [safe_dict(x) for x in d]
    else:
        return d


def catchKeyboardInterrupt(func):
    def dec(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except KeyboardInterrupt, _e:
            print 'KeyboardInterrupt --> %s cancelled' % func
    return dec


def canonalize_uri(uriref):
    """Returns Uri that is valid or raises Exception"""
    if not uriref:
        raise UriNotValid("uri is None")

    # TODO: if I would implement canonalization following rfc (cut port,
    # lowercase domain, http and https is equal then I couldn't compare graphs
    # anymore

    if isinstance(uriref, rdflib.BNode):
        return uriref

    assert not isinstance(uriref, rdflib.Literal)

    if not uriref.encode('utf8'):
        raise UriNotValid("Not valid: %s" % uriref)

    if not hasattr(uriref, "n3"):
        logger.debug(u"Not an URIRef: %s, fixing that." % uriref)
        uriref = rdflib.URIRef(uriref)

    if uriref.startswith('#'):
        raise UriNotValid("%s starts with '#'. Check your Parser" % uriref)

    return uriref


def hash_to_slash_uri(uri):
    """Converts Hash to Slash uri http://www.w3.org/wiki/HashURI"""
    # TODO: replace with urlparse
    assert isinstance(uri, rdflib.URIRef)
    if '#' in uri:
        real_uri, hash_value = uri.split("#")
        assert real_uri
        return real_uri
    return uri


def reverse_dict(dct):
    # TODO rdflib.URIRef dict?
    res = {}
    for k,v in dct.iteritems():
        res[v] = k
    return safe_dict(res)


def predicate2pyattr(predicate, namespace_short_notation_reverse_dict):
    prefix, propertyname = split_uri(predicate)
    assert prefix
    assert propertyname
    #if not "_" in propertyname:
    #    logger.info("%s_%s may cause problems?" % (prefix, propertyname))
    if not prefix in namespace_short_notation_reverse_dict:
        logger.warning("%s cannot be shortened" % predicate)
        return predicate

    if namespace_short_notation_reverse_dict[prefix] == "":
        return propertyname
    else:
        return u"%s_%s" % (namespace_short_notation_reverse_dict[prefix],
                       propertyname)


def pyattr2predicate(pyattr, namespace_dict):
    # TODO: build urirefdict instead of unicodedict?

    if pyattr.startswith(u"http://"):
        return rdflib.URIRef(pyattr)

    splitlist = pyattr.split("_")

    # HACK: are there constrains for namespace prefixes?
    splitlistlen = len(splitlist)
    if splitlistlen == 1:
        # attribute "homepage" --> check if "" in namespace_dict
        prefix = ""
        property_name = splitlist[0]
    elif (splitlistlen > 2 and
        u"_".join(splitlist[0:2]) in namespace_dict):
        # http://www.geonames.org/ontology# defines 'wgs84_pos' --> \
        # 'wgs84_pos_lat' cannot be solved with approach we took until now
        prefix = u"_".join(splitlist[0:2])
        property_name = u"_".join(splitlist[2:])
        assert prefix, pyattr
    else:
        prefix = splitlist[0]
        property_name = u"_".join(splitlist[1:])
        assert prefix, pyattr

    assert property_name, pyattr

    # timbl foaf defines foaf as "" --> mbox_sha1sum makes problems here
    if "" in namespace_dict:
        if not prefix in namespace_dict:
            logger.error("problem. %s, %s" % (prefix, pyattr))
            return rdflib.URIRef(u"%s%s"
        % (namespace_dict[""], pyattr))
    else:
        assert namespace_dict[prefix], (u"%s not in namespace_dict. Trying to "
            "solve %s") % (prefix, pyattr)

    return rdflib.URIRef(u"%s%s"
        % (namespace_dict[prefix], property_name))


from backends import *
from models import *

######### Resource #########

class ResourceManager(Manager):

    def get_pk(self, origin_uri, uri):
        return origin_uri+uri

    def create(self, uri, origin, **kwargs):
        assert isinstance(origin, Origin), "Origin instance required"
        #assert origin.processed, ("Origin has to be processed before creating "
        #    "more Resource objects: origin.GET()")

        if uri.startswith("#"):
            assert not isinstance(uri, rdflib.BNode), \
                "bnode should not start with #"
            uri = rdflib.URIRef(origin.uri + uri)

        uri = canonalize_uri(uri)
        pk = self.get_pk(origin_uri=origin.uri, uri=uri)

        obj = super(ResourceManager, self).create(
            pk=pk, _uri=uri, _origin=origin, **kwargs
        )

        # move to tests
        if isinstance(uri, rdflib.BNode):
            assert isinstance(obj._uri, rdflib.BNode)

        assert not hasattr(obj, "_has_changes")

        return obj


    def get(self, uri, origin=None, return_authoritative_resource=True):
        """If the authoratative Origin to the Resource does not exist and no
        origin is given then DoesNotExist is returned. Assumption is
        to only trust validated sources.

        Alternative: this could point to source with most parameters given
        etc because user may want to just have the Resource with most
        content/know everything about a uri.
        --> If DoesNotExist occurs Resources with uri might still
        exist but no validated Resources exist.
        """
        uri = canonalize_uri(uri)

        if not origin:
            filter_result = list(self.filter(_uri=uri))

            if not filter_result:
                raise self.model.DoesNotExist

            if len(filter_result) == 1:
                # return only match
                return filter_result[0]
            else:
                if return_authoritative_resource:
                    # try to return best match
                    authoritative_resource = None
                    for resource in filter_result:
                        if uri.startswith(resource._origin.uri):
                            authoritative_resource = resource
                            break
                    if authoritative_resource:
                        return authoritative_resource
                    else:
                        raise self.model.DoesNotExist("No authoritative "
                            "Resource found for %s" %uri)
                else:
                    raise self.model.DoesNotExist("Please pass the exact "
                        "Origin. The Resource you are looking for is "
                        "provided by the Origins: %s"
                        % ", ".join([unicode(r._origin.uri)
                                     for r in filter_result]))

        assert isinstance(origin, Origin), origin
        pk = self.get_pk(origin_uri=origin.uri, uri=uri)
        return super(ResourceManager, self).get(pk=pk)
        assert 0, "implement!"

    def get_or_create(self, uri, origin=None, auto_origin=False):
        uri = canonalize_uri(uri)
        if auto_origin:
            assert not origin, "Either origin or auto_origin, not both"
            origin_uri = hash_to_slash_uri(uri)
            origin, _origin_created = Origin.objects\
                .get_or_create(uri=origin_uri)

        try:
            return self.get(uri=uri, origin=origin), False
        except self.model.DoesNotExist:
            return self.create(uri=uri, origin=origin), True


class Resource(Model):

    _uri = URIRefField()
    _origin = ObjectField()
    objects = ResourceManager()

    def __unicode__(self):
        str = u"%s" % self._uri
        if hasattr(self, 'foaf_name'):
            str += u' "%s"' % unicode(self.foaf_name)
            if len(self.foaf_name) > 1: str += u",..."
        if hasattr(self, "_origin") and isinstance(self._origin, Origin):
            str += u" [%r]" % self._origin.uri.encode("utf8")
        return str

    def get_attributes(self):
        dct = copy.copy(self.__dict__)
        for attr in [u"pk",
            u"_origin",
            u"_uri",
            u"_reverse",
            u"_has_changes"]:
            if hasattr(self, attr):
                dct.pop(attr)
        return dct

    def _add_property(self, predicate, object,
                      namespace_short_notation_reverse_dict):
        assert isinstance(predicate, rdflib.URIRef),\
            "Not an URIRef: %s"%predicate
        assert hasattr(predicate, "n3"),\
            "property %s is not a rdflib object" % predicate

        # add Resource object directly instead of uriref
        # hash function is important for that!
        is_resource = False
        if isinstance(object, rdflib.BNode):
            is_resource = True
        elif isinstance(object, rdflib.URIRef):
            if not isinstance(object, rdflib.Literal):
                o = urlparse(object)
                if o.scheme == "http":
                    is_resource = True
                else:
                    logger.debug("Not a Resource URI because not valid: %s "
                                 "--> should be rdflib.Literals?" % object)

        predicate = predicate2pyattr(predicate,
                                     namespace_short_notation_reverse_dict)

        if is_resource:
            logger.debug("%s . %s = Resource( %s )"
                         % (self._uri, predicate, object))
            object, _created = Resource.objects.get_or_create(
                uri=object, origin=self._origin)

            if not hasattr(object, "_reverse"):
                object._reverse = {}
            if predicate in object._reverse:
                if not isinstance(object._reverse[predicate], set):
                    object._reverse[predicate] = set([object._reverse[predicate]])
                object._reverse[predicate].add(self)
            else:
                object._reverse[predicate] = self

        if hasattr(self, predicate):
            attr = getattr(self, predicate)
            if not type(attr) == set:
                setattr(self, predicate, set([attr]))
                attr = getattr(self, predicate)
            attr.add(object)
            assert object in getattr(self, predicate)
        else:
            setattr(self, predicate, object)
            attr = getattr(self, predicate)
            #assert attr == object

        # TODO: this might cause UnicodeDecodeErrors
        #logger.debug(u"Contribute_to_object %s: %s = %s"
        #    % (self, predicate.encode('utf8'), attr))
        
        # TODO: introduce rdflib.Literal dict or force_unicode when
        # reconverting from __dict__?
        assert str(predicate) in self.__dict__

    def _tripleserialize_iterator(self,
                                  namespace_dict):
        for property, values in self.__dict__.items():

            # skip internals
            if str(property).startswith("_") or property == "pk":
                continue

            if property.startswith("http://"):
                property = rdflib.URIRef(property)
            else:
                property = pyattr2predicate(property, namespace_dict)

            assert hasattr(property, "n3"), \
                "property %s is not a rdflib object" % property

            if isinstance(values, set):
                for v in values:
                    if isinstance(v, self.__class__):
                        # If object is referenced in attribute, "de-reference"
                        yield((self._uri, property, v._uri))
                    else:
                        yield((self._uri, property, v))
            else:
                v = values
                if isinstance(v, self.__class__):
                    # If object is referenced in attribute, "de-reference"
                    yield((self._uri, property, v._uri))
                else:
                    yield((self._uri, property, v))


    def __setattr__(self, key, value):
        # TODO: is there a better way to validate attributes?
        if key in self._meta.fields:
            field = self._meta.fields.get(key)
            if field and value:
                value = field.to_python(value)
        elif not key.startswith("_") and not key == "pk": # TODO: rename to_pk?
            # rdf properties here
            self._has_changes = True
        Model.__setattr__(self, key, value)

    def delete(self):
        if hasattr(self, "_has_changes"):
            assert not self._has_changes
        assert hasattr(self, "pk") and self.pk is not None
        # TODO: use Collector objects as django does
        self.__class__.objects._storage.__delitem__(self.pk)

    def save(self):
        # TODO: introduce "clean" to make sure only valid properties modified
        created = not self.pk
        if created:
            assert 0, ("Please use Resource.objects.create() to create Resource"
                       "objects!")
            #assert not self.__class__.objects.get(self.uri, self.origin)
            #self = self.__class__.objects.create(**self.get_attributes())

        # TODO: write tests for the following lines
        assert self in Resource.objects.filter(_origin=self._origin)

        if self in Resource.objects.filter(_origin=self._origin,
                _has_changes=True):
            assert len(list(Resource.objects.filter(_origin=self._origin,
                _has_changes=True))) == 1
        else:
            logging.info("New resource object was created")

        #values = dict((name, getattr(self, name)) for name in \
        #        self._meta.fields.iterkeys())
        self.update() #**values)

    def update(self):
        self._origin.PUT()
        self._has_changes = False


######### Origin #########

class OriginManager(Manager):

    def create_hook(self, uri):
        assert not '#' in uri, ("HashURI not allowed as Origin: %s. Maybe "
            "you are looking for "
            "Resource.objects.get_or_create(...,auto_origin=True)?" % uri)

    def create(self, uri, BACKEND=None):
        uri = canonalize_uri(uri)
        self.create_hook(uri)
        backend = BACKEND if BACKEND else RestBackend()
        return super(OriginManager, self).create(pk=uri, uri=uri,
                                                 backend=backend)

    def get(self, uri, **kwargs):
        """Retrieves Origin object from Store"""
        uri = canonalize_uri(uri)
        return super(OriginManager, self).get(pk=uri)

    def get_or_create(self, uri, **kwargs):
        uri = canonalize_uri(uri)
        assert str(uri) == str(hash_to_slash_uri(uri))
        try:
            if kwargs:
                logger.warning("kwargs not supportet in get")
            return self.get(uri), False
        except self.model.DoesNotExist:
            return self.create(uri, **kwargs), True

    @catchKeyboardInterrupt
    def GET_all(self, depth=2, **kwargs):
        """Crawls or Re-Crawls all Origins. Passes Arguments to GET"""
        # TODO: limit crawling speed
        assert depth<5
        func = lambda origin: True if not origin.processed else False
        for _i in range(depth):
            crawl = filter(func, self.all())
            if crawl:
                for origin in crawl:
                    origin.GET(raise_errors=False, **kwargs)

class Origin(Model):

    uri = URIRefField()
    objects = OriginManager()
    backend = ObjectField()
    
    def add_error(self, error):
        if not hasattr(self, 'errors'): self.errors = []
        self.errors.append(error)

    def __init__(self, pk=None, **kwargs):
        super(Origin, self).__init__(pk=pk, **kwargs)
        self.stats = {}
        self.processed = False

    def __unicode__(self):
        str = u"%s" % unicode(self.uri)
        str += " %s" % self.backend.__class__.__name__
        if hasattr(self, 'errors'):
            for error in self.errors:
                str += u" %s" % error
        if self.processed:
            str += u" Processed"
        return str

    def GET(self,
            GRAPH_SIZE_LIMIT=30000,
            only_follow_uris=None,
            handle_owl_imports=False,
            skip_urls=None,
            raise_errors=True,
            ):

        if not self.uri:
            raise Exception("Please provide URI first")

        logger.info(u"GET %s..." % self.uri)

        if self.has_unsaved_changes():
            if self.processed:
                raise Exception("Please save all changes before querying "
                                "again. Merging not supported yet")
            else:
                logger.warning("There were Resource objects created before "
                               "processing the resource's origin.")

        if skip_urls is not None and str(self.uri) in skip_urls:
            self.add_error("Skipped")
            self.processed = True
            return

        self.stats['last_processed'] = datetime.datetime.now()

        try:
            data = self.backend.GET(self.uri)
        except urllib2.HTTPError as e:
            if e.code in [
                    401,
                    403,
                    503, # Service Temporarily Unavailable
                    404, # Not Found
                    ]:
                self.add_error(e.code)
            if raise_errors: raise e
            else: return
        except urllib2.URLError as e:
            self.add_error("timeout")
            if raise_errors: raise e
            else: return

        if not data:
            self.processed = True
            return

        assert self.backend.format, "format is needed later"

        graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)

        try:
            # Important: Do not pass data=data without publicID=uri because
            # relative URIs (#deri) won't be an absolute uri in that case!
            assert data
            graph.parse(data=data, publicID=self.uri,
                        format=self.backend.format)
        except SAXParseException:
            self.add_error("SAXParseException")
            logger.error("SAXParseException: %s" % self)
            if raise_errors: raise e
            else: return
        except rdflib.exceptions.ParserError:
            self.add_error("ParserError")
            logger.error("ParserError: %s" % self)
            if raise_errors: raise e
            else: return
        except IOError as e:
            # TODO: why does this occur? guess: wrong protocol
            self.add_error("IOError")
            logger.error("IOError: %s" % self)
            if raise_errors: raise e
            else: return

        self.processed = True
        if not graph:
            logger.warning("%s did not return any information" % self.uri)
            return

        if hasattr(self, "errors"): delattr(self, "errors")

        g_length = len(graph)
        if g_length > 5000:
            logger.warning("len(graph) == %s" % g_length)
            if g_length > GRAPH_SIZE_LIMIT:
                logger.error("Maximum graph size is set to %s. The aquired "
                             "graph exceeds that! Pass "
                             "GRAPH_SIZE_LIMIT to set it differently."
                             % GRAPH_SIZE_LIMIT);
                return

        # normal rdflib.compare does not work correctly with
        # conjunctiveGraph, unless there is only one graph within that
        assert len(list(graph.contexts())) == 1

        if hasattr(self, "_graph"):
            # we already assured that there are no unsaved_changes
            # --> graph() == _graph
            logger.info(u"Already crawled: %s. Comparing graphs..." % self.uri)

            if not compare.to_isomorphic(self._graph) == \
                   compare.to_isomorphic(graph):
                logging.warning("GET retrieved updates for %s!" % self.uri)
                utils.my_graph_diff(self._graph, graph)

                for resource in self.get_resources():
                    resource.delete()

                delattr(self, "handled")

        if hasattr(self, "handled"):
            return

        self._graph = graph

        if not list(self.get_resources()):
            logger.debug("Resources exist but no _graph --> Resources were "
                         "created locally")

        def create_origin(o, caused_by=None, # should be Origin object
                                process_now=False):
            if not type(o) == rdflib.URIRef:
                return

            uri = hash_to_slash_uri(o)
            origin, created = Origin.objects.get_or_create(uri=uri)
            if created:
                setattr(origin, '_created_by', caused_by)
            if process_now:
                logger.info("Interrupting to load %s because we need to "
                        "process owl:imports %s first" % (caused_by.uri,
                                                          origin.uri))
                origin.GET()

                
        namespace_short_notation_reverse_dict = reverse_dict(dict(graph\
            .namespace_manager.namespaces()))

        # TODO: pass only_follow_uris = [] to skip creating origins? --> find
        # a better way
        if only_follow_uris is not None:
            only_follow_uris = [rdflib.URIRef(u) if not\
                isinstance(u, rdflib.URIRef) else u for u in only_follow_uris]

        start_time = datetime.datetime.now()

        for subject, predicate, obj_ect in graph:
            assert hasattr(subject, "n3")
            #subject = canonalize_uri(subject)

            assert predicate.encode('utf8')

            if handle_owl_imports:
                if predicate == rdflib.OWL.imports:
                    create_origin(obj_ect, caused_by=self, process_now=True)

            if only_follow_uris is not None:
                if predicate in only_follow_uris:
                    create_origin(obj_ect, caused_by=self)
            else:
                # follow every URIRef! this could take a long time!
                create_origin(obj_ect, caused_by=self)

            resource, _created = Resource.objects.get_or_create(uri=subject,
                                                                origin=self)
            resource._add_property(predicate, obj_ect,
                                   namespace_short_notation_reverse_dict)

        self.stats['graph_processing_time'] = datetime.datetime.now() - start_time

        for resource in self.get_resources():
            resource._has_changes = False

        assert compare.to_isomorphic(graph) == \
               compare.to_isomorphic(self.graph()), \
               utils.my_graph_diff(graph, self.graph())

        self.handled = True


    def get_resources(self):
        return Resource.objects.filter(_origin=self)

    def graph(self):
        """Processes every Resource and Property related to 'self' and
        creates rdflib.ConjunctiveGraph because rdflib.Graph does not allow
        parsing plugins
        """
        # TODO: rdflib.graph() ?
        graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)

        if not hasattr(self, '_graph'):
            if len(self.errors) == 0:
                self.GET(raise_errors=False) # TODO: test for recursion?
            else:
                logging.error("Origin %s has Errors --> can't process .graph()"
                    % self.uri)
                return graph

        # TODO: find a better way to do this
        # Problems:
        #  1) namespace bindings are not really necessary to validate
        #     isomorphic graphs but the resulting graph is is different
        #     if they miss
        #  2) doesn't detect duplicate definitions of namespaces
        namespace_dict = dict(self._graph.namespace_manager.namespaces())

        for prefix, namespace in safe_dict(namespace_dict).items():
            graph.bind(prefix=prefix, namespace=namespace)
        new_ns = dict(graph.namespace_manager.namespaces())

        assert namespace_dict == new_ns, [(k, v)
            for k, v in safe_dict(namespace_dict).items() \
                if not k in safe_dict(new_ns).keys()]

        for resource in self.get_resources():
            # TODO: better idea how to do this?
            # __dict__ converts rdflib.urirefs to strings -->
            # converts back to uriref
            # {'foaf': 'http:/....', ...}
            for triple in resource._tripleserialize_iterator(namespace_dict):
                graph.add(triple)
        return graph

    def has_unsaved_changes(self):
        # resource objects exist although not processed yet
        if not self.processed and len(list(self.get_resources())):
            return True

        # objects with changed attributes exist
        if any(resource._has_changes
                for resource in self.get_resources()\
                    if hasattr(resource, '_has_changes')):
            return True

        return False

    def PUT(self):
        assert self.processed
        if hasattr(self, "errors"):
            assert not self.errors, ("There were errors fetching the "
                                     "resource. PUT not possible")

        if not self.has_unsaved_changes():
            logging.error("Nothing to PUT for %s!" % self.uri)
            return

        graph = self.graph()
        data = graph.serialize(format=self.backend.format)

        self.backend.PUT(data=data)
        # TODO return "OK"

    def get_statistics(self):
        def triples_per_second(triples, time): # TODO make this more accurate
            total_seconds = (time.microseconds+(time.seconds+\
                                      time.days*24*3600)*10**6)//10**6
            return triples / total_seconds if total_seconds > 0 else None

        if hasattr(self, '_graph'):
            triples = len(self._graph)
            tps = triples_per_second(triples,
                                     self.stats['graph_processing_time'])
            if tps:
                logger.info(
                    "Crawled %s: '%s' triples in '%s' seconds --> '%s' "
                    "triples/second"
                    % (self.uri, triples,
                       self.stats['graph_processing_time'], tps))
            else:
                logger.info("Crawled %s in '%s' seconds"
                % (self.uri, self.stats['graph_processing_time']))
            pass
        else:
            print "no statistics recorded"



def check_shortcut_consistency():
    """Checks every known Origin for inconsistent namespacemappings"""
    global_namespace_dict = {}
    for origin in Origin.objects.all():
        if hasattr(origin, "_graph"):
            for k, v in dict(origin._graph.namespace_manager\
                        .namespaces()).items():
                if k in global_namespace_dict:
                    assert global_namespace_dict[k] == v
                else:
                    global_namespace_dict[k] = v



import rdflib
from rdflib import compare
import logging

logger = logging.getLogger("ldtools")


def my_graph_diff(graph1, graph2):
    """Compares graph2 to graph1 and highlights everything that changed.
    Colored if pygments available"""

    import difflib

    # quick fix for wrong type
    if not type(graph1) == type(graph2) == rdflib.Graph:
        if type(graph1) == rdflib.ConjunctiveGraph:
            g1contexts = list(graph1.contexts())
            assert len(g1contexts) == 1
            graph1 = g1contexts[0]
        if type(graph2) == rdflib.ConjunctiveGraph:
            g2contexts = list(graph2.contexts())
            assert len(g2contexts) == 1
            graph2 = g2contexts[0]


    # Return if both graphs are isomorphic
    iso1 = compare.to_isomorphic(graph1)
    iso2 = compare.to_isomorphic(graph2)

    if graph1.identifier == graph2.identifier:
        str_bit = u"The 2 '%s' Graphs" % graph1.identifier
    else:
        str_bit = (u"Graphs '%s' and '%s'"
                   % (graph1.identifier, graph2.identifier))

        # TODO remove later
        assert not are_isomorphic(graph1, graph2)

    if iso1 == iso2:
        logger.debug(u"%s are isomorphic" % str_bit)
        return

    print u"Differences between %s." % str_bit

    print u"Graph1:",len(graph1),u"Graph2:",len(graph2)

    in_both, in_first, in_second = compare.graph_diff(iso1, iso2)

    def dump_nt_sorted(g):
        return sorted(g.serialize(format='nt').splitlines())

    sorted_first = dump_nt_sorted(in_first)
    sorted_second = dump_nt_sorted(in_second)

    diff = difflib.unified_diff(
        sorted_first,
        sorted_second,
        u'Original',
        u'Current',
        lineterm=''
    )

    try:
        from pygments import highlight
        from pygments.formatters import terminal
        from pygments.lexers import web

        lexer = web.XmlLexer()
        formatter = terminal.TerminalFormatter()
        print highlight(u'\n'.join(diff), lexer, formatter)
    except ImportError:
        logger.info("Install pygments for colored diffs")
        print u'\n'.join(diff)
    except UnicodeDecodeError:
        print u"Only in first", unicode(sorted_first)
        print u"Only in second", unicode(sorted_second)


class Myobject(object):
    def __hash__(self):
        print "__hash__", self
        return hash(self.pk)
    def __eq__(self, other):
        print "__eq__", self, other
        return (type(other) == type(self) and
                other.__dict__.keys() == self.__dict__.keys() and
                all(getattr(other, key) == getattr(self, key)
                    for key in self.__dict__.keys()))
    def __ne__(self, other):
        print "__ne__", self, other
        if type(other) != type(self): return True
        if any(getattr(other, key) != getattr(self, key) \
                   for key in self.__dict__.keys()):
            return True

class HashAndEqTest(): #unittest.TestCase):
    def test_it(self):
        a = Myobject()
        a.pk = 1
        b = Myobject()
        b.pk = 1
        b.anotherattr = "value"
        myset = set()
        myset.add(a)
        myset.add(b)

        assert len(myset) == 1, len(myset)
        assert a != b



# -*- coding: utf-8 -*-
import copy
import ldtools as models
import unittest2
import rdflib


class Sample1(models.Model):
    attr1 = models.StringField()

class TestModelInit1(unittest2.TestCase):
    def test_wrong_init1(self):
        self.assertRaises(KeyError, Sample1, {"a":"b"})

    def test_wrong_init2(self):
        self.assertRaises(KeyError, Sample1, {"attr1":"a","a":"b"})

    def test_init(self):
        self.assertIn('pk', Sample1(attr1="test").__dict__.keys())
        

class Sample2(models.Model):
    attr1 = models.StringField()
    attr2 = models.StringField()

class TestModelHashAndEq(unittest2.TestCase):
    def setUp(self):
        self.p1 = Sample2(attr1=u"hü", attr2=u"vä")
        self.p2 = Sample2(attr1=u"hü", attr2=u"vä")
        self.p3 = Sample2(attr1=u"hüa", attr2=u"vä")

    def test_hash1(self):
        sett=set([self.p1, self.p2])
        self.assertIn(self.p1, sett)

    def test_hash2(self):
        self.assertNotIn(self.p3, set([self.p1, self.p2]))

    def test_equivalence1(self):
        self.assertEquals(self.p1, self.p2)

    def test_equivalence2(self):
        self.assertNotEquals(self.p1, self.p3)


class Sample3(models.Model):
    attr1 = models.StringField()
    objects = models.Manager()

class TestManager(unittest2.TestCase):
    def setUp(self):
        Sample3.objects.reset_store()

    def create_one(self):
        pk = attr1 = u"tü"
        self.o = Sample3.objects.create(pk=pk, attr1=attr1)

    def test_create(self):
        self.create_one()
        assert self.o in Sample3.objects.all()

    def test_filter1(self):
        pk = attr1 = u"tü"
        o = Sample3.objects.create(pk=pk, attr1=attr1)
        filter_result = Sample3.objects.filter_has_key("attr1")
        self.assert_(filter_result)
        self.assertEqual(len(filter_result), 1)
        self.assertIn(o, filter_result)

    def test_filter2(self):
        pk = attr1 = u"tü"
        o = Sample3.objects.create(pk=pk, attr1=attr1)
        filter_result = list(Sample3.objects.filter(attr1=attr1))
        self.assert_(filter_result)
        self.assertEqual(len(filter_result), 1)
        self.assertIn(o, filter_result)

    def test_filter3(self):
        pk = attr1 = u"tü"
        o = Sample3.objects.create(pk=pk, attr1=attr1)
        filter_result = list(Sample3.objects.filter(attr1="a"))
        self.assertEqual(len(filter_result), 0)
        self.assertEqual(filter_result, [])

    def test_filter3(self):
        o1, o2, o3 = self._create_objects(u"a", u"b", u"c")

        self.assertEqual(len(Sample3.objects.all()), 3)
        filter_result = list(Sample3.objects.filter(attr1="b"))
        self.assertEqual(len(filter_result), 1)
        self.assertIn(o2, filter_result)

    def test_filter_attr_check(self):
        o1, o2, o3 = self._create_objects(u"a", u"b", u"c")

        filter_result = list(Sample3.objects.filter(attr1="b"))
        self.assertEqual(len(filter_result), 1)
        self.assertNotIn(o1, filter_result)
        self.assertNotIn(o3, filter_result)

    def _create_objects(self, *args):
        obj_list = []
        for string in args:
            pk = attr1 = string
            obj_list.append(Sample3.objects.create(pk=pk, attr1=attr1))
        return obj_list

    def test_filter_allows_filtering_runtime_parameters1(self):
        o1, o2 = self._create_objects(u"a", u"b")
        o1.processed=True

        filter_result = list(Sample3.objects.filter(processed=True))
        self.assertEqual(len(filter_result), 1)
        self.assertIn(o1, filter_result)
        self.assertNotIn(o2, filter_result)

    def test_filter_allows_filtering_runtime_parameters2(self):
        o1, o2, o3 = self._create_objects(u"a", u"b", u"c")
        o1.processed=True
        o3.processed=True

        filter_result = list(Sample3.objects.filter(processed=True))
        self.assertEqual(len(filter_result), 2)
        self.assertIn(o1, filter_result)
        self.assertIn(o3, filter_result)
        self.assertNotIn(o2, filter_result)

class Sample4(models.Model):
    uri = models.URIRefField()

class TestModelURIRefField(unittest2.TestCase):
    def test_init(self):
        self.assertIn('pk',
                      Sample4(uri=rdflib.URIRef("test")).__dict__.keys())


class ModelEqualityAndHashFunctionTest(unittest2.TestCase):
    def setUp(self):
        Sample3.objects.reset_store()

    def test_is_equal_1(self):
        o1 = Sample3.objects.create(pk=1, attr1="test")
        o2 = Sample3.objects.create(pk=2, attr1="test")
        # dirty trick to trick equality function
        o2.pk = 1

        self.assertEqual(o1, o2)

    def test_is_equal_2(self):
        o1 = Sample3.objects.create(pk=1, attr1="test")
        o2 = Sample3.objects.create(pk=2, attr1="test2")
        # dirty trick to trick equality function
        o2.pk = 1

        self.assert_(not o1 == o2)

    def test_are_not_equal_1(self):
        o1 = Sample3.objects.create(pk=1, attr1="test")
        o2 = copy.copy(o1)
        o1.another_attr = "val"

        self.assertNotEqual(o1, o2)

        # TODO: __hash__ should make the following statement true but it doesnt
        # test_set = set()
        #test_set.add(o1)
        #test_set.add(o2)
        #self.assertEqual(o1.pk, o2.pk)
        #self.assertEqual(len(test_set), 1)

    def test_are_not_equal_3(self):
        o1 = Sample3.objects.create(pk=1, attr1="test1")
        o2 = Sample3.objects.create(pk=2, attr1="test2")
        self.assertNotEqual(o1, o2)


if __name__ == '__main__':
    unittest2.main()


# -*- coding: utf-8 -*-
__version__ = "0.4.4"
__useragent__ = ('ldtools-%s (http://github.com/dmr/ldtools, daniel@nwebs.de)'
                 % __version__)

import datetime
import logging
import mimetypes
import os
import rdflib
import shutil
import urllib2
import glob

# add n3 to known mimetypes
mimetypes.add_type("text/n3", ".n3")

logger = logging.getLogger("ldtools")


def get_file_extension(filename):
    """
    >>> get_file_extension("test.xml")
    'xml'
    >>> get_file_extension("test.1234123.xml")
    'xml'
    >>> get_file_extension("test")
    ''
    """
    extension = filename.split(".")[1:][-1:]
    return str(extension[0]) if extension else ""

def assure_parser_plugin_exists(format):
    # TODO: double check if parser for format exists -->
    # TODO: move parser to backend
    try:
        rdflib.graph.plugin.get(name=format,
                                kind=rdflib.parser.Parser)
        return True
    except rdflib.plugin.PluginException as e:
        logger.error("Parser does not exist for format %s" % format)
        return False

def guess_format_from_filename(file_name):
    file_extension = file_name.split(".")[-1]
    if file_name != file_extension:
        return file_extension

def build_filename_from_uri(uri):
    file_name = uri.lstrip("http://").replace(".","_")\
        .replace("/","__").replace("?","___").replace("&","____")

    # TODO: hacky
    folder = os.path.abspath("cache")
    if not os.path.exists(folder):
        os.mkdir(folder)

    return os.path.join(folder, file_name)




class Backend(object):
    """ Abstract Backend to demonstrate API
    """
    def GET(self, uri):
        raise NotImplementedError
    def PUT(self):
        raise NotImplementedError


class RestBackend(Backend):

    def GET(self, uri):
        """lookup URI"""
        # TODO: friendly crawling: use robots.txt speed limitation definitions

        if not hasattr(self, "uri"):
            self.uri = uri
        else:
            if not self.uri == uri:
                raise Exception("You cannot pass different uris to the same "
                                "backend")

        headers = {'User-agent': __useragent__,
                   'Accept':('application/rdf+xml,text/rdf+n3;q=0.9,'
                             'application/xhtml+xml;q=0.5, */*;q=0.1')}

        request = urllib2.Request(url=self.uri, headers=headers)

        opener = urllib2.build_opener() #SmartRedirectHandler())
        result_file = opener.open(request)

        self.content_type = result_file.headers['Content-Type'].split(";")[0]

        # Many servers don't do content negotiation: if one of the following
        # content_types are returned by server, assume the mapped type
        content_type_mapping = {
            "text/plain": "application/rdf+xml"
        }

        if self.content_type in content_type_mapping:
            self.content_type = content_type_mapping[self.content_type]

        file_extension = mimetypes.guess_extension(self.content_type)
        if file_extension:
            format = file_extension.strip(".")
            if format in ["rdf", "ksh"]:
                # fix responses that we know are wrong
                format = "xml"

            if assure_parser_plugin_exists(format):
                self.format = format

                data = result_file.read()

                return data
        else:
            #logger.warning("%s not supported by ldtools. Trying 'xml'..."
            #               % result_file.headers['Content-Type'])
            #self.format = "xml"
            #self.content_type = mimetypes.types_map[".%s" % self.format]
            # TODO: assumption here: empty string will not cause harm because
            # Origin.GET() will break after this
            return

    def PUT(self, data):
        assert self.uri, "GET has to be called before PUT possible"

        # TODO: authentication? oauth?
        #h = httplib2.Http()
        #h.add_credentials('name', 'password')
        #resp, content = h.request(uri, "PUT", body=data, headers=headers)
        #if resp.status != 200: raise Error(resp.status, errmsg, headers)
        #return resp, content
        headers={"content-type": self.content_type,
                 "User-Agent": __useragent__,
                 "Content-Length": str(len(data))}

        opener = urllib2.build_opener(urllib2.HTTPHandler)
        request = urllib2.Request(self.uri,
                                  data=data,
                                  headers=headers)
        request.get_method = lambda: 'PUT'
        response = opener.open(request)


class FileBackend(Backend):
    """Manages one xml file --> Uri that the user wants to "PUT" to is not
    flexible!
    """

    def __init__(self, filename, format=None):
        assert os.path.exists(filename)

        if format:
            if assure_parser_plugin_exists(format):
                self.format = format
            else:
                logger.error("No format set!")
        else:
            format = guess_format_from_filename(filename)
            if format and assure_parser_plugin_exists(format):
                self.format = format
            else:
                logger.error("No format set!")

        self.filename = filename

    def GET(self, uri):
        if not hasattr(self, "uri"):
            self.uri = uri
        else:
            if not self.uri == uri:
                raise Exception("You cannot pass different uris to the same "
                                "backend")

        with open(self.filename, "r") as f:
            data = f.read()
        return data

    def PUT(self, data):
        assert self.uri, "GET has to be called before PUT possible"

        if os.path.exists(self.filename):
            # File already exists. Make backup copy
            now = datetime.datetime.strftime(datetime.datetime.utcnow(),
                                             '%Y%m%d-%H%M%S')
            file_extension = get_file_extension(self.filename)
            if file_extension:
                old_version = u"%s.%s.%s" % (
                    self.filename.strip(file_extension),
                    now, file_extension)
            else:
                old_version = u"%s_%s" % (self.filename, now)
            shutil.copy(self.filename, old_version)

        with open(self.filename, "w") as f:
            f.write(data)
        self.old_version = old_version

    def revert_to_old_version(self):
        if hasattr(self, "old_version"):
            logger.info("Reverting to version before last saved version")
            shutil.copy(self.old_version, self.filename)
            os.remove(self.old_version)
            delattr(self, "old_version")


class MemoryBackend(Backend):
    def __init__(self, data=None, format="xml"):
        self.data = data if data else ""
        self.format = format

    def GET(self, uri):
        return self.data

    def PUT(self, data):
        self.data = data


class CachedRestBackend(RestBackend):
    @staticmethod
    def _write_to_cache(uri, format, data):
        file_name = build_filename_from_uri(uri)
        file_name = u"%s.%s" % (file_name, format)
        assert not os.path.exists(file_name), file_name
        with open(file_name, "w") as f:
            f.write(data)

    @staticmethod
    def _get_from_cache(uri):
        # build_filename_from_uri returns dbpedia__org__karlsruhe --> check
        # if dbpedia__org__karlsruhe.n3 exists
        guessed_file_name = build_filename_from_uri(uri)
        print guessed_file_name
        file_name_glob_list = glob.glob(guessed_file_name + "*")
        print file_name_glob_list
        if file_name_glob_list:
            if len(file_name_glob_list) == 1:
                file_name = file_name_glob_list[0]
            else:
                # dbpedia_org__resource__Category:Karlsruhe.xml und
                # dbpedia_org__resource__Category:Karlsruhe_region.xml
                file_name = min(file_name_glob_list, key=len)
            format = guess_format_from_filename(file_name)
            if format and assure_parser_plugin_exists(format):
                format = format
                with open(file_name, "r") as f:
                    data = f.read()
                return data, format
        return False, False

    def _pre_GET(self):
        cached_file, format = self._get_from_cache(self.uri)
        if cached_file:
            self.format = format
            return cached_file

    def _post_GET(self):
        self._write_to_cache(uri=self.uri, format=self.format,
                                     data=data)


# -*- coding: utf-8 -*-
import ldtools
import os
import rdflib
import unittest2
from rdflib import compare


class OriginGETAndPUT(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

        filename = "www_w3_org__People__Berners-Lee__card.xml"
        file_name = os.path.join(os.path.dirname(__file__), filename)
        backend = ldtools.FileBackend(file_name)

        uri = "http://xmlns.com/foaf/0.1/"
        #uri = "http://www.w3.org/People/Berners-Lee/card"

        self.origin1 = ldtools.Origin.objects.create(uri, BACKEND=backend)
        self.origin1.GET()

        self.resource1 = ldtools.Resource.objects.get(
            uri="http://xmlns.com/foaf/0.1/PersonalProfileDocument",
            origin=self.origin1)

        results = list(ldtools.Resource.objects.filter(_origin=self.origin1,
            rdf_type=self.resource1))

        for result in results:
            self.assertEqual(result.rdf_type, self.resource1)

        self.assertEqual(results[0].dc_title,
             rdflib.Literal(u"Tim Berners-Lee's editable FOAF file"))

        self.resource2 = results[0]

    def test_set_attribute(self):
        self.resource2.dc_title = rdflib.Literal(u"TEST")

        self.assert_(self.resource2._has_changes)
        self.assertEqual(self.resource2.dc_title, rdflib.Literal(u"TEST"))
        self.resource2.save()
        self.assert_(not self.resource2._has_changes)

    def tearDown(self):
        self.resource2._origin.backend.revert_to_old_version()


if __name__ == '__main__':
    unittest2.main()


# -*- coding: utf-8 -*-
import ldtools
import rdflib
import unittest2
from rdflib import compare


class OriginIsDirtyTest(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

    def test_resource_created_for_origin_causes_unsaved(self):
        origin = ldtools.Origin.objects.create("http://example.org/",
            BACKEND=ldtools.MemoryBackend())

        self.assert_(not origin.has_unsaved_changes())

        origin.GET()

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 0)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)
        self.assert_(origin.processed)

        resource = ldtools.Resource.objects.create("http://example.org/test1",
            origin=origin)

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 1)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)

        self.assert_(origin.has_unsaved_changes())

    def test_resource_created_for_origin_saved_does_not_cause_unsaved(self):
        origin = ldtools.Origin.objects.create("http://example.org/",
            BACKEND=ldtools.MemoryBackend())

        self.assert_(not origin.has_unsaved_changes())

        origin.GET()

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 0)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)
        self.assert_(origin.processed)

        resource = ldtools.Resource.objects.create("http://example.org/test1",
            origin=origin)
        resource.save()

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 1)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)

        self.assert_(not origin.has_unsaved_changes())


class AcceptanceTests(unittest2.TestCase):
    def setUp(self):
        ldtools.Origin.objects.reset_store()
        ldtools.Resource.objects.reset_store()

    def test_create_origin(self):
        origin = ldtools.Origin.objects.create("http://example.org/",
            BACKEND=ldtools.MemoryBackend())

        origin.GET()

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 0)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)
        self.assert_(origin.processed)

        resource = ldtools.Resource.objects.create("http://example.org/test1",
            origin=origin)

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 1)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)

        resource._origin.GET()

        self.assertEqual(len(list(ldtools.Resource.objects.all())), 1)
        self.assertEqual(len(list(ldtools.Origin.objects.all())), 1)
